-------------------------------------------------------------------------------
--                L  O  C  A  L     V  A  R  I  A  B  L  E  S
-------------------------------------------------------------------------------

local addonName, L = ...;

local NUM_MOUNTS_PER_ROW  = 6;
local NUM_MOUNTS_PER_PAGE = 12;

local mounts =
{
  ["run"]  = {},
  ["fly"]  = {},
  ["swim"] = {}
};

local mountRetype =
{
  [75207] = 0x04, -- Abyssal Seahorse
  [75614] = 0x03, -- Celestial Steed
  [44153] = 0x03, -- Flying Machine
  [48025] = 0x07, -- Headless Horseman's Mount
  [30174] = 0x04, -- Riding Turtle
  [64731] = 0x04, -- Sea Turtle
  [75973] = 0x07  -- X-53 Touring Rocket
};

local mountSpells =
{
  [ 1066] = 0x04, -- Aquatic Form
  [33943] = 0x02, -- Flight Form
  [ 2645] = 0x01, -- Ghost Wolf
  [87840] = 0x01, -- Running Wild
  [40120] = 0x02, -- Swift Flight Form
  [  783] = 0x01, -- Travel Form
};

local mountRestricted =
{
  [75207] = -- Abyssal Seahorse
  {
    [L["Shimmering Expanse"]] = true,
    [L["Abyssal Depths"]]     = true,
    [L["Kelp'thar Forest"]]   = true
  },
  [25953] = -- Blue Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = true
  },
  [26056] = -- Green Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = true
  },
  [26054] = -- Red Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = true
  },
  [26055] = -- Yellow Qiraji Battle Tank
  {
    [L["Ahn'Qiraj"]] = true
  }
};

local continentalFlight = {};

local isTravelForm = {};
local mountMacroText = "";

-------------------------------------------------------------------------------
--                 L  O  C  A  L     F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

-------------------------------------------------------------------------------
--                G  L  O  B  A  L    F  U  N  C  T  I  O  N  S
-------------------------------------------------------------------------------

--
-- Function clear the macro of the hidden mount button after it has been
-- 'clicked'.
--
function NigglesMountPostClick(button)
  -- Clear the button's macro
  if (InCombatLockdown() == nil) then
    button:SetAttribute("macrotext", mountMacroText);
  end
end

--
-- Function to work out what to do when a macro calls '/click NigglesMount'.
-- This function is called before the button is 'clicked'. It works out what
-- to do based on the player's current status and changes the macro of the
-- hidden button accordingly.
--
function NigglesMountPreClick(button)
  -- Local Variables
  local currentContinent;
  local currentZone;
  local formIdx = GetShapeshiftForm();
  local isWintergraspBattle = false;
  local mapId;
  local mountType = nil;
  local randomList = {{}, {}};
  local spellName = "";
  local wintergraspName;

  -- Get the current continent
  mapId = GetCurrentMapAreaID();
  SetMapToCurrentZone();
  currentContinent = GetCurrentMapContinent();
  SetMapByID(mapId);

  -- Get the name of the current zone
  currentZone = GetRealZoneText();

  -- Get info about Wintergrasp
  _, wintergraspName, isWintergraspBattle = GetWorldPVPAreaInfo(1);

  -- Work out what to do, based on the player's current location and state
  if (UnitInVehicle("player")) then
    -- Exit any vehicle the player is in
    VehicleExit();
  elseif (IsMounted() == 1) then
    -- Dismount from any mount the player is on
    Dismount();
  elseif ((formIdx > 0) and
          (isTravelForm[formIdx] == true) and
          (InCombatLockdown() == nil)) then
    -- Cancel the current shapeshift form
    button:SetAttribute("macrotext", "/cancelform [form:"..formIdx.."]");
  elseif ((not IsModifierKeyDown()) and
          (IsSwimming())) then
    -- If swimming mounts have been selected...
    if (next(NigglesDBC.mounts.swim) ~= nil) then
      -- ...use that type of mounts
      mountType = "swim";
    else
      mountType = "run";
    end
  elseif ((not IsModifierKeyDown()) and
          (IsFlyableArea())) then
    -- Work out if the player can fly
    if ((continentalFlight[currentContinent] == true) and
        ((currentContinent ~= 4) or
         (currentZone ~= wintergraspName) or
         (isWintergraspBattle ~= true))) then
      mountType = "fly";
    else
      mountType = "run";
    end
  else
    mountType = "run";
  end

  -- Check if the player is in combat
  if (mountType ~= nil) then
    if (InCombatLockdown() == 1) then
      UIErrorsFrame:AddMessage(SPELL_FAILED_AFFECTING_COMBAT,
        1.0, 0.1, 0.1, 1.0);
    elseif (next(NigglesDBC.mounts[mountType]) == nil) then
      UIErrorsFrame:AddMessage(L["No mounts selected"], 1.0, 0.1, 0.1, 1.0);
    else
      -- Build restricted & unrestricted lists of selected mounts
      for spellId, _ in pairs(NigglesDBC.mounts[mountType]) do
        if (mountRestricted[spellId] ~= nil) then
          if (mountRestricted[spellId][currentZone] == true) then
            table.insert(randomList[1], spellId);
          end
        else
          table.insert(randomList[2], spellId);
        end
      end

      -- Randomly select a mount, prioritizing those restricted to current zone
      if (#(randomList[1]) > 0) then
        spellName = GetSpellInfo(randomList[1][math.random(#(randomList[1]))]);
      elseif (#(randomList[2]) > 0) then
        spellName = GetSpellInfo(randomList[2][math.random(#(randomList[2]))]);
      end

      -- Update the button with the mount's spell name
      if (spellName ~= "") then
        button:SetAttribute("macrotext",
          mountMacroText.."/cast "..spellName);
      else
        button:SetAttribute("macrotext", "");
        UIErrorsFrame:AddMessage(L["No suitable mount"], 1.0, 0.1, 0.1, 1.0);
      end
    end
  end

  return;
end

--
-- Function to initialise a Mounts frame
--
function NigglesMountsFrameInit(frame, mountType, text, tooltip)
  -- Save the mount type and mask
  frame.mountType = mountType;

  -- Initialise the text
  _G[frame:GetName().."LabelText"]:SetText(L[text]);
  _G[frame:GetName().."Label"].tooltip = L[tooltip];

  return;
end

--
-- Function to get info about all the mounts
--
function NigglesMountsGetInfo()
  -- Local Variables
  local creatureId;
  local creatureName;
  local creatureSpellId;
  local icon;
  local mountType;
  local numMounts = GetNumCompanions("MOUNT");

  -- Clear the lists of mounts
  wipe(mounts["run"]);
  wipe(mounts["fly"]);
  wipe(mounts["swim"]);

  -- Add any know mount spells
  for spellId, spellMask in pairs(mountSpells) do
    -- Check if the spell is know
    if (IsSpellKnown(spellId)) then
      -- Get the spell's icon
      _, _, icon = GetSpellInfo(spellId);

      -- Add the spell to the appropriate list(s)
      if (bit.band(spellMask, 0x01) ~= 0) then
        table.insert(mounts["run"],
          {
            ["spellId"] = spellId,
            ["icon"]    = icon
          });
      end
      if (bit.band(spellMask, 0x02) ~= 0) then
        table.insert(mounts["fly"],
          {
            ["spellId"] = spellId,
            ["icon"]    = icon
          });
      end
      if (bit.band(spellMask, 0x0C) ~= 0) then
        table.insert(mounts["swim"],
          {
            ["spellId"] = spellId,
            ["icon"]    = icon
          });
      end
    end
  end

  -- Add any know mounts
  for mountIdx = 1, numMounts do
    -- Get the mount's info
    creatureId, creatureName, creatureSpellId, icon, _, mountType =
      GetCompanionInfo("MOUNT", mountIdx);

    -- Adjust the mount's type
    if (mountRetype[creatureSpellId] ~= nil) then
      mountType = mountRetype[creatureSpellId];
    else
      if (bit.band(mountType, 0x02) ~= 0) then
        mountType = 0x02;
      else
        mountType = 0x01;
      end
    end

    -- Add the mount to the appropriate list(s)
    if (bit.band(mountType, 0x01) ~= 0) then
      table.insert(mounts["run"],
        {
          ["creatureId"]   = creatureId,
          ["creatureName"] = creatureName,
          ["spellId"]      = creatureSpellId,
          ["icon"]         = icon
        });
    end
    if (bit.band(mountType, 0x02) ~= 0) then
      table.insert(mounts["fly"],
        {
          ["creatureId"]   = creatureId,
          ["creatureName"] = creatureName,
          ["spellId"]      = creatureSpellId,
          ["icon"]         = icon
        });
    end
    if (bit.band(mountType, 0x0C) ~= 0) then
      table.insert(mounts["swim"],
        {
          ["creatureId"]   = creatureId,
          ["creatureName"] = creatureName,
          ["spellId"]      = creatureSpellId,
          ["icon"]         = icon
        });
    end
  end

  return;
end

--
-- Function to update the mounts displayed by a frame in the Niggles Options
-- Mounts panel.
--
function NigglesMountsListUpdate(frame)
  -- Local Variables
  local button;
  local frameName;
  local mountIdx;
  local numMounts;
  local scrollbar;
  local topLine;

  -- Check the required variables are initialised
  if ((Niggles.updatedOptions ~= nil) and
      (Niggles.updatedOptions.mounts ~= nil) and
      (frame.mountType ~= nil) and
      (Niggles.updatedOptions.mounts[frame.mountType] ~= nil)) then
    -- Get the frame's name
    frameName = frame:GetName();

    -- Get the number of mounts of the appropriate type
    numMounts = #(mounts[frame.mountType]);

    -- Get the frame's scrollbar
    scrollbar = _G[frameName.."CompanionsScrollbar"];

    -- Make sure the top line displayed is still valid
    topLine = FauxScrollFrame_GetOffset(scrollbar);
    if (topLine > math.ceil(numMounts/NUM_MOUNTS_PER_ROW)) then
      -- Work out the new top line
      topLine = math.ceil(numMounts/NUM_MOUNTS_PER_ROW)-1;
      if (topLine < 0) then
        topLine = 0;
      end

      -- Set the new top line
      FauxScrollFrame_SetOffset(scrollbar, topLine);
    end

    -- Update all the buttons
    for buttonIdx = 1, NUM_MOUNTS_PER_PAGE do
      -- Get the current button
      button = _G[frameName.."Companions"..buttonIdx];

      -- Work out the index of the mount to display in the button
      mountIdx = (topLine*NUM_MOUNTS_PER_ROW)+buttonIdx;
      if (mountIdx <= numMounts) then
        -- Update the button
        button.creatureId   = mounts[frame.mountType][mountIdx].creatureId;
        button.creatureName = mounts[frame.mountType][mountIdx].creatureName;
        button.spellId      = mounts[frame.mountType][mountIdx].spellId;
        button:SetNormalTexture(mounts[frame.mountType][mountIdx].icon);
        if (Niggles.updatedOptions.mounts[frame.mountType][button.spellId] ~=
             nil) then
          button:SetChecked(true);
        else
          button:SetChecked(false);
        end
        button:Enable();
      else
        -- Update the button
        button:SetChecked(false);
        button:Disable();
      end
    end

    -- Update the scrollbar
    FauxScrollFrame_Update(scrollbar,
      math.ceil(numMounts/NUM_MOUNTS_PER_ROW), 2, 37,
      nil, 0, 0, nil, 0, 0, true);
  end

  return;
end

--
-- Function to scroll the list of mounts
--
function NigglesMountsListScroll(frame)
  NigglesMountsListUpdate(frame:GetParent());

  return;
end

--
-- Function to process clicks on check buttons in the Niggles Options
-- Mount panel.
--
function NigglesMountClicked(checkButton)
  -- Local Variables
  local mountType;

  -- Check the button has the required info
  if ((checkButton ~= nil) and
      (checkButton.spellId ~= nil)) then
    -- Get the type of mount the check button is for
    mountType = (checkButton:GetParent()):GetParent().mountType;
    if (Niggles.updatedOptions.mounts[mountType] ~= nil) then
      Niggles.updatedOptions.mounts[mountType][checkButton.spellId] =
        checkButton:GetChecked();
    end
  end

  return;
end

--
-- Function called when the player's spells change
--
function NigglesMountsSpellsUpdate()
  -- Local Variables
  local formName;
  local numContinents = select("#", GetMapContinents());
  local numForms = GetNumShapeshiftForms();
  local spellName;

  -- Clear the continental flight table
  for idx = 1, numContinents do
    continentalFlight[idx] = false;
  end

  -- Check if the player has the skill to fly
  if ((IsSpellKnown(34090)) or    -- Expert Riding
      (IsSpellKnown(34091)) or    -- Artisan Riding
      (IsSpellKnown(90265))) then -- Master Riding
    -- Check if the player can fly in Azeroth
    if (IsSpellKnown(90267)) then -- Flight Pilot's License
      continentalFlight[1] = true;
      continentalFlight[2] = true;
      continentalFlight[5] = true;
    end

    -- Set the flag that indicates the player can fly in Outland
    continentalFlight[3] = true;

    -- Check if the player can fly in Northrend
    if (IsSpellKnown(54197)) then -- Cold Weather Flying
      continentalFlight[4] = true;
    end
  end

  -- Work out which forms are travel forms
  wipe(isTravelForm);
  for idx, spellId in ipairs(mountSpells) do
    spellName = GetSpellInfo(spellId);
    if (spellName ~= nil) then
      for formIdx = 1, numForms do
        _, formName = GetShapeshiftFormInfo(formIdx);
        if ((formName ~= nil) and
            (formName == spellName)) then
          isTravelForm[formIdx] = true;
          break;
        end
      end
    end
  end

  -- Build the macro text for the mount button
  mountMacroText = "";
  for idx = 1, numForms do
    if (isTravelForm[idx] == true) then
      mountMacroText = mountMacroText.."[combat,form:"..idx.."]";
    else
      isTravelForm[idx] = false;
    end
  end
  if (mountMacroText ~= "") then
    mountMacroText = "/cancelform "..mountMacroText.."\n";
  end

  return
end

--
-- Function to refresh the settings for the Niggles Options Mounts panel.
--
function NigglesOptionsRefreshMounts()
  -- Set the scroll frames to the top
  FauxScrollFrame_OnVerticalScroll(
    NigglesOptionsMountsRunCompanionsScrollbar,  0, 37, nil);
  NigglesMountsListUpdate(NigglesOptionsMountsRun);
  FauxScrollFrame_OnVerticalScroll(
    NigglesOptionsMountsFlyCompanionsScrollbar,  0, 37, nil);
  NigglesMountsListUpdate(NigglesOptionsMountsFly);
  FauxScrollFrame_OnVerticalScroll(
    NigglesOptionsMountsSwimCompanionsScrollbar, 0, 37, nil);
  NigglesMountsListUpdate(NigglesOptionsMountsSwim);

  return;
end
